@@ -208,14 +208,14 @@ void FltkViewport::draw ()
    if (d) {
       draw_area(this, x(), y(), w () - hdiff, h () - vdiff);
 
-      if (d == FL_DAMAGE_CHILD) {
-         if (hscrollbar->damage ())
-            draw_child (*hscrollbar);
-         if (vscrollbar->damage ())
-            draw_child (*vscrollbar);
-      } else {
+      if (d == FL_DAMAGE_ALL || hscrollbar->damage ())
          draw_child (*hscrollbar);
+      if (d == FL_DAMAGE_ALL || vscrollbar->damage ())
          draw_child (*vscrollbar);
+
+      if (d == FL_DAMAGE_ALL && hdiff && vdiff) {
+         fl_color(FL_BACKGROUND_COLOR);
+         fl_rectf(x()+w()-hdiff, y()+h()-vdiff, hdiff, vdiff);
       }
    }
 
@@ -233,9 +233,9 @@ int FltkViewport::handle (int event)
        * sends the event here. Returning zero tells FLTK to resend the
        * event as SHORTCUT, which we finally route to the parent. */
 
-      /* As we don't know the exact keybindings set by the user, we ask
-       * for all of them (except Tab to keep form navigation). */
-      if (Fl::event_key() != FL_Tab)
+      /* As we don't know the exact keybindings set by the user, we ask for
+       * all of them (except for the minimum needed to keep form navigation).*/
+      if (Fl::event_key() != FL_Tab || Fl::event_ctrl())
          return 0;
       break;
 
@@ -275,7 +275,9 @@ int FltkViewport::handle (int event)
       break;
 
    case FL_DRAG:
-      if (dragScrolling && Fl::event_button() == FL_MIDDLE_MOUSE) {
+      if (Fl::event_inside(this))
+         Fl::remove_timeout(selectionScroll);
+      if (dragScrolling) {
          scroll(dragX - Fl::event_x(), dragY - Fl::event_y());
          dragX = Fl::event_x();
          dragY = Fl::event_y();
@@ -286,6 +288,11 @@ int FltkViewport::handle (int event)
       } else if (horScrolling) {
          hscrollbar->handle(event);
          return 1;
+      } else if (!Fl::event_inside(this)) {
+         mouse_x = Fl::event_x();
+         mouse_y = Fl::event_y();
+         if (!Fl::has_timeout(selectionScroll, this))
+            Fl::add_timeout(0.025, selectionScroll, this);
       }
       break;
 
@@ -294,6 +301,7 @@ int FltkViewport::handle (int event)
       break;
 
    case FL_RELEASE:
+      Fl::remove_timeout(selectionScroll);
       if (Fl::event_button() == FL_MIDDLE_MOUSE) {
          setCursor (core::style::CURSOR_DEFAULT);
       } else if (verScrolling) {
@@ -332,7 +340,8 @@ void FltkViewport::setCanvasSize (int width, int ascent, int descent)
  */
 void FltkViewport::positionChanged ()
 {
-   if (mouse_x != -1 && dragScrolling == false)
+   if (!dragScrolling && mouse_x >= x() && mouse_x < x()+w() && mouse_y >= y()
+       && mouse_y < y()+h())
       (void)theLayout->motionNotify (this,
                                      translateViewXToCanvasX (mouse_x),
                                      translateViewYToCanvasY (mouse_y),
@@ -425,6 +434,32 @@ void FltkViewport::scroll (core::ScrollCommand cmd)
    }
 }
 
+/*
+ * Scrolling in response to selection where the cursor is outside the view.
+ */
+void FltkViewport::selectionScroll ()
+{
+   int distance;
+   int dx = 0, dy = 0;
+
+   if ((distance = x() - mouse_x) > 0)
+      dx = -distance * hscrollbar->linesize () / 48 - 1;
+   else if ((distance = mouse_x - (x() + w())) > 0)
+      dx = distance * hscrollbar->linesize () / 48 + 1;
+   if ((distance = y() - mouse_y) > 0)
+      dy = -distance * vscrollbar->linesize () / 48 - 1;
+   else if ((distance = mouse_y - (y() + h())) > 0)
+      dy = distance * vscrollbar->linesize () / 48 + 1;
+
+   scroll (dx, dy);
+}
+
+void FltkViewport::selectionScroll (void *data)
+{
+   ((FltkViewport *)data)->selectionScroll ();
+   Fl::repeat_timeout(0.025, selectionScroll, data);
+}
+
 void FltkViewport::setViewportSize (int width, int height,
                                     int hScrollbarThickness,
                                     int vScrollbarThickness)
