@@ -38,6 +38,18 @@ ImageMapsList::ImageMap::~ImageMap ()
    delete shapesAndLinks;
 }
 
+void ImageMapsList::ImageMap::draw (core::View *view,core::style::Style *style,
+                                    int x, int y)
+{
+   container::typed::Iterator <ShapeAndLink> it;
+
+   for (it = shapesAndLinks->iterator (); it.hasNext (); ) {
+      ShapeAndLink *shapeAndLink = it.getNext ();
+
+      shapeAndLink->shape->draw(view, style, x, y);
+   }
+}
+
 void ImageMapsList::ImageMap::add (core::Shape *shape, int link) {
    ShapeAndLink *shapeAndLink = new ShapeAndLink ();
    shapeAndLink->shape = shape;
@@ -105,6 +117,15 @@ void ImageMapsList::setCurrentMapDefaultLink (int link)
    currentMap->setDefaultLink (link);
 }
 
+void ImageMapsList::drawMap (lout::object::Object *key, core::View *view,
+                             core::style::Style *style, int x, int y)
+{
+   ImageMap *map = imageMaps->get (key);
+
+   if (map)
+      map->draw(view, style, x, y);
+}
+
 int ImageMapsList::link (object::Object *key, int x, int y)
 {
    int link = -1;
@@ -136,7 +157,7 @@ Image::Image(const char *altText)
 Image::~Image()
 {
    if (altText)
-      delete altText;
+      free(altText);
    if (buffer)
       buffer->unref ();
    if (mapKey)
@@ -228,8 +249,9 @@ void Image::enterNotifyImpl (core::EventCrossing *event)
    currLink = getStyle()->x_link;
 
    if (currLink != -1) {
-      (void) emitLinkEnter (currLink, -1, -1, -1);
+      (void) layout->emitLinkEnter (this, currLink, -1, -1, -1);
    }
+   Widget::enterNotifyImpl(event);
 }
 
 void Image::leaveNotifyImpl (core::EventCrossing *event)
@@ -238,25 +260,53 @@ void Image::leaveNotifyImpl (core::EventCrossing *event)
 
    if (currLink != -1) {
       currLink = -1;
-      (void) emitLinkEnter (-1, -1, -1, -1);
+      (void) layout->emitLinkEnter (this, -1, -1, -1, -1);
    }
+   Widget::leaveNotifyImpl(event);
+}
+
+/*
+ * Return the coordinate relative to the contents.
+ * If the event occurred in the surrounding box, return the value at the
+ * edge of the contents instead.
+ */
+int Image::contentX (core::MousePositionEvent *event)
+{
+   int ret = event->xWidget - getStyle()->boxOffsetX();
+
+   ret = misc::min(getContentWidth(), misc::max(ret, 0));
+   return ret;
+}
+
+int Image::contentY (core::MousePositionEvent *event)
+{
+   int ret = event->yWidget - getStyle()->boxOffsetY();
+
+   ret = misc::min(getContentHeight(), misc::max(ret, 0));
+   return ret;
 }
 
 bool Image::motionNotifyImpl (core::EventMotion *event)
 {
-   if (mapList) {
-      /* client-side image map */
-      int newLink = mapList->link (mapKey, event->xWidget, event->yWidget);
-      if (newLink != currLink) {
-         currLink = newLink;
-         clicking = false;
-         setCursor(newLink == -1 ? core::style::CURSOR_DEFAULT :
-                                   core::style::CURSOR_POINTER);
-         (void) emitLinkEnter (newLink, -1, -1, -1);
+   if (mapList || isMap) {
+      int x = contentX(event);
+      int y = contentY(event);
+
+      if (mapList) {
+         /* client-side image map */
+         int newLink = mapList->link (mapKey, x, y);
+         if (newLink != currLink) {
+            currLink = newLink;
+            clicking = false;
+            /* \todo Using MAP/AREA styles would probably be best */
+            setCursor(newLink == -1 ? getStyle()->cursor :
+                                      core::style::CURSOR_POINTER);
+            (void) layout->emitLinkEnter (this, newLink, -1, -1, -1);
+         }
+      } else if (isMap && currLink != -1) {
+         /* server-side image map */
+         (void) layout->emitLinkEnter (this, currLink, -1, x, y);
       }
-   } else if (isMap && currLink != -1) {
-      /* server-side image map */
-      (void) emitLinkEnter (currLink, -1, event->xWidget, event->yWidget);
    }
    return true;
 }
@@ -264,10 +314,12 @@ bool Image::motionNotifyImpl (core::EventMotion *event)
 bool Image::buttonPressImpl (core::EventButton *event)
 {
    bool ret = false;
-   currLink = mapList ? mapList->link (mapKey, event->xWidget, event->yWidget):
+
+   currLink = mapList ? mapList->link(mapKey,contentX(event),contentY(event)) :
       getStyle()->x_link;
    if (event->button == 3){
-      (void)emitLinkPress(currLink, getStyle()->x_img, -1,-1,event);
+      (void)layout->emitLinkPress(this, currLink, getStyle()->x_img, -1, -1,
+                                  event);
       ret = true;
    } else if (event->button == 1 || currLink != -1){
       clicking = true;
@@ -278,13 +330,13 @@ bool Image::buttonPressImpl (core::EventButton *event)
 
 bool Image::buttonReleaseImpl (core::EventButton *event)
 {
-   currLink = mapList ? mapList->link (mapKey, event->xWidget, event->yWidget):
+   currLink = mapList ? mapList->link(mapKey,contentX(event),contentY(event)) :
       getStyle()->x_link;
    if (clicking) {
-      int x = isMap ? event->xWidget : -1;
-      int y = isMap ? event->yWidget : -1;
+      int x = isMap ? contentX(event) : -1;
+      int y = isMap ? contentY(event) : -1;
       clicking = false;
-      emitLinkClick (currLink, getStyle()->x_img, x, y, event);
+      layout->emitLinkClick (this, currLink, getStyle()->x_img, x, y, event);
       return true;
    }
    return false;
@@ -311,34 +363,48 @@ void Image::draw (core::View *view, core::Rectangle *area)
                           intersection.x - dx, intersection.y - dy,
                           intersection.width, intersection.height);
    } else {
+      core::View *clippingView;
+
       if (altText && altText[0]) {
+         core::View *usedView = view;
+
+         clippingView = NULL;
+
          if (altTextWidth == -1)
             altTextWidth =
                layout->textWidth (getStyle()->font, altText, strlen (altText));
 
-         core::View *clippingView = NULL, *usedView = view;
-         if (allocation.width < altTextWidth ||
-             allocation.ascent < getStyle()->font->ascent ||
-             allocation.descent < getStyle()->font->descent) {
+         if ((getContentWidth() < altTextWidth) ||
+             (getContentHeight() <
+              getStyle()->font->ascent + getStyle()->font->descent)) {
             clippingView = usedView =
                view->getClippingView (allocation.x + getStyle()->boxOffsetX (),
                                       allocation.y + getStyle()->boxOffsetY (),
-                                      allocation.width
-                                      - getStyle()->boxDiffWidth (),
-                                      allocation.ascent + allocation.descent
-                                      - getStyle()->boxDiffHeight ());
+                                      getContentWidth(),
+                                      getContentHeight());
          }
 
-         usedView->drawText (getStyle()->font, getStyle()->color,
+         usedView->drawSimpleWrappedText (getStyle()->font, getStyle()->color,
                              core::style::Color::SHADING_NORMAL,
                              allocation.x + getStyle()->boxOffsetX (),
-                             allocation.y + getStyle()->boxOffsetY ()
-                             + getStyle()->font->ascent,
-                             altText, strlen(altText));
+                             allocation.y + getStyle()->boxOffsetY (),
+                             getContentWidth(), getContentHeight(), altText);
 
          if (clippingView)
             view->mergeClippingView (clippingView);
       }
+      if (mapKey) {
+         clippingView = view->getClippingView (allocation.x +
+                                               getStyle()->boxOffsetX (),
+                                               allocation.y +
+                                               getStyle()->boxOffsetY (),
+                                               getContentWidth(),
+                                               getContentHeight());
+         mapList->drawMap(mapKey, clippingView, getStyle(),
+                          allocation.x + getStyle()->boxOffsetX (),
+                          allocation.y + getStyle()->boxOffsetY ());
+         view->mergeClippingView (clippingView);
+      }
    }
 
    /** TODO: draw selection */
