@@ -37,7 +37,7 @@ Iterator::Iterator(Widget *widget, Content::Type mask, bool atEnd)
    this->mask = mask;
 }
 
-Iterator::Iterator(Iterator &it): object::Object (), misc::Comparable ()
+Iterator::Iterator(Iterator &it): object::Comparable ()
 {
    widget = it.widget;
    content = it.content;
@@ -205,7 +205,7 @@ object::Object *EmptyIterator::clone ()
    return new EmptyIterator (*this);
 }
 
-int EmptyIterator::compareTo (misc::Comparable *other)
+int EmptyIterator::compareTo (object::Comparable *other)
 {
    EmptyIterator *otherIt = (EmptyIterator*)other;
 
@@ -257,7 +257,7 @@ TextIterator::TextIterator (TextIterator &it): Iterator (it)
    text = it.text;
 }
 
-int TextIterator::compareTo (misc::Comparable *other)
+int TextIterator::compareTo (object::Comparable *other)
 {
    TextIterator *otherIt = (TextIterator*)other;
 
@@ -535,7 +535,7 @@ object::Object *DeepIterator::clone ()
    return it;
 }
 
-int DeepIterator::compareTo (misc::Comparable *other)
+int DeepIterator::compareTo (object::Comparable *other)
 {
    DeepIterator *otherDeepIterator = (DeepIterator*)other;
 
@@ -665,7 +665,7 @@ object::Object *CharIterator::clone()
    return cloned;
 }
 
-int CharIterator::compareTo(misc::Comparable *other)
+int CharIterator::compareTo(object::Comparable *other)
 {
    CharIterator *otherIt = (CharIterator*)other;
    int c = it->compareTo(otherIt->it);
@@ -794,5 +794,5 @@ void CharIterator::unhighlight (CharIterator *it1, CharIterator *it2,
    }
 }
 
-} // namespace dw
 } // namespace core
+} // namespace dw
