@@ -40,16 +40,20 @@ ListItem::~ListItem()
 void ListItem::initWithWidget (core::Widget *widget,
                                 core::style::Style *style)
 {
+   hasListitemValue = true;
    addWidget (widget, style);
    addSpace (style);
-   updateValue ();
+   if (style->listStylePosition == core::style::LIST_STYLE_POSITION_OUTSIDE)
+      updateValue ();
 }
 
 void ListItem::initWithText (const char *text, core::style::Style *style)
 {
+   hasListitemValue = true;
    addText (text, style);
    addSpace (style);
-   updateValue ();
+   if (style->listStylePosition == core::style::LIST_STYLE_POSITION_OUTSIDE)
+      updateValue ();
 }
 
 int ListItem::getValue ()
@@ -57,7 +61,7 @@ int ListItem::getValue ()
    if (words->size () == 0)
       return 0;
    else
-      return words->get(0).size.width + words->get(0).origSpace;
+      return words->getRef(0)->size.width + words->getRef(0)->origSpace;
 }
 
 void ListItem::setMaxValue (int maxValue, int value)
