@@ -75,7 +75,7 @@ const char *const AboutSplash=
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
 "    <td>\n"
-"     <a href='http://hg.dillo.org/dillo/file/tip/ChangeLog'>\n"
+"     <a href='http://hg.dillo.org/dillo/raw-file/tip/ChangeLog'>\n"
 "     ChangeLog</a>\n"
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
@@ -121,10 +121,6 @@ const char *const AboutSplash=
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
 "    <td>\n"
-"     <a href='http://www.linux.org.uk/Portaloo.cs'>Linux.org.uk</a>\n"
-"    <tr>\n"
-"    <td>&nbsp;&nbsp;\n"
-"    <td>\n"
 "     <a href='http://www.commondreams.org/'>C.&nbsp;Dreams</a>\n"
 "    <tr>\n"
 "    <td>&nbsp;&nbsp;\n"
@@ -234,22 +230,32 @@ const char *const AboutSplash=
 "<tr>\n"
 " <td bgcolor='#CCCCCC'>\n"
 "  <h4>Release overview</h4>\n"
-"  December 05, 2011\n"
+"  April 17, 2013\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'>\n"
 "  <tr>\n"
 "   <td>\n"
 "<p>\n"
-"dillo-3.0.2 brings you some new bits and pieces, as listed below :)\n"
+"dillo-3.0.3 adds some nice new features, as listed below.\n"
 "<p>\n"
-"After this release, the core team plans to focus on implementing the CSS\n"
-"feature of floating elements. This will <em>greatly</em> improve dillo's\n"
-"web page rendering since many sites have adopted floats instead of tables.\n"
+"This release comes with <b>configurable UI colors</b> in\n"
+"<a href='http://www.dillo.org/dillorc'>dillorc</a>.\n"
+"Packagers should set the theme that better fits the distro, and\n"
+"end users may tweak freely if they like!\n"
 "<p>\n"
-"Dillo3 uses the FLTK GUI toolkit's 1.3.x series. In June, fltk-1.3.0 was\n"
-"<a href='http://fltk.org/articles.php?L1086'>released</a>,\n"
-"clearing the way for Dillo to return to those distributions\n"
+"In order to use the <b>hyphenation</b> feature, pattern files from CTAN\n"
+"need to be installed. Most probably your packager did it for you. If not\n"
+"enabled, just follow directions from the README file.\n"
+"<p>\n"
+"A new <a href='http://www.dillo.org/domainrc'>domainrc</a> mechanism has\n"
+"been introduced to provide finer-grained control over automatic requests\n"
+"for third-party resources, and the old filter_auto_requests dillorc option\n"
+"has been removed. This makes it easier to block ads and trackers but permit\n"
+"legitimate use such as wikipedia pulling images from wikimedia.org.\n"
+"<p>\n"
+"Dillo3 uses the FLTK GUI toolkit's 1.3.x series, whose release\n"
+"cleared the way for Dillo to return to those distributions\n"
 "which had excluded Dillo2 due to FLTK2 never being officially released.\n"
 "<p>\n"
 "The core team welcomes developers willing to join our workforce.\n"
@@ -266,7 +272,7 @@ const char *const AboutSplash=
 " <td bgcolor='#CCCCCC'>\n"
 "  <h4>ChangeLog highlights</h4>\n"
 "  (Extracted from the\n"
-"  <a href='http://hg.dillo.org/dillo/file/tip/ChangeLog'>full\n"
+"  <a href='http://hg.dillo.org/dillo/raw-file/tip/ChangeLog'>full\n"
 "  ChangeLog</a>)\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
@@ -274,11 +280,10 @@ const char *const AboutSplash=
 "  <tr>\n"
 "   <td>\n"
 "<ul>\n"
-"<li>Digest authentication.\n"
-"<li>Rework line breaking and fix white-space:nowrap handling.\n"
-"<li>text-transform property.\n"
-"<li>Locale-independent ASCII character case handling (fixes Turkic locales).\n"
-"<li>Bind Ctrl-{PageUp,PageDown} to tab-{previous,next}.\n"
+"<li>Automatic hyphenation (includes penalty_* preferences that control line-breaking).\n"
+"<li>Added the domainrc mechanism.\n"
+"<li>Added user interface color preferences (ui_*).\n"
+"<li>Support for CSS display property.\n"
 "</ul>\n"
 "  </table>\n"
 "</table>\n"
