@@ -8,15 +8,11 @@
  */
 #define BW_Root            (1)  /* Root URLs */
 #define BW_Img             (2)  /* Image URLs */
-#define Bw_Force           (4)  /* Stop connection too */
-
-
-typedef struct _BrowserWindow BrowserWindow;
+#define BW_Force           (4)  /* Stop connection too */
 
 
 /* browser_window contains the specific data for a single window */
-struct _BrowserWindow
-{
+typedef struct {
    /* Pointer to the UI object this bw belongs to */
    void *ui;
 
@@ -46,12 +42,9 @@ struct _BrowserWindow
    /* 'nav_stack_ptr' refers to what's being displayed */
    int nav_stack_ptr;        /* [0 based; -1 = empty] */
    /* When the user clicks a link, the URL isn't pushed directly to history;
-    * nav_expect_url holds it until the first answer-bytes are got. Only then
-    * it is sent to history and referenced at the top of nav_stack */
+    * nav_expect_url holds it until a dw is assigned to it. Only then an entry
+    * is made in history and referenced at the top of nav_stack */
    DilloUrl *nav_expect_url;
-   /* 'nav_expecting' is true while dillo waits for non-cached URL data,
-    * until a dw is assigned to it. */
-   bool_t nav_expecting;
 
    /* Counter for the number of hops on a redirection. Used to stop
     * redirection loops (accounts for WEB_RootUrl only) */
@@ -64,7 +57,7 @@ struct _BrowserWindow
    /* HTML-bugs detected at parse time */
    int num_page_bugs;
    Dstr *page_bugs;
-};
+} BrowserWindow;
 
 
 #ifdef __cplusplus
@@ -88,6 +81,11 @@ void *a_Bw_get_url_doc(BrowserWindow *bw, const DilloUrl *Url);
 void a_Bw_remove_doc(BrowserWindow *bw, void *vdoc);
 void a_Bw_add_url(BrowserWindow *bw, const DilloUrl *Url);
 void a_Bw_cleanup(BrowserWindow *bw);
+/* expect API */
+void a_Bw_expect(BrowserWindow *bw, const DilloUrl *Url);
+void a_Bw_cancel_expect(BrowserWindow *bw);
+bool_t a_Bw_expecting(BrowserWindow *bw);
+const DilloUrl *a_Bw_expected_url(BrowserWindow *bw);
 
 typedef void (*BwCallback_t)(BrowserWindow *bw, const void *data);
 
