@@ -8,7 +8,7 @@
 extern "C" {
 #endif /* __cplusplus */
 
-int32_t a_Color_parse (const char *subtag, int32_t default_color, int *err);
+int32_t a_Color_parse (const char *str, int32_t default_color, int *err);
 int32_t a_Color_vc(int32_t candidate, int32_t c1, int32_t c2, int32_t c3);
 
 #ifdef __cplusplus
