@@ -1,7 +1,6 @@
 /*
  * File: jpeg.c
  *
- * Copyright (C) 1997 Raph Levien <raph@acm.org>
  * Copyright (C) 2000-2007 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -47,20 +46,22 @@ typedef enum {
    DILLO_JPEG_ERROR
 } DilloJpegState;
 
+typedef struct DilloJpeg DilloJpeg;
+
 /* An implementation of a suspending source manager */
 
 typedef struct {
    struct jpeg_source_mgr pub;  /* public fields */
-   struct DilloJpeg *jpeg;      /* a pointer back to the jpeg object */
+   DilloJpeg *jpeg;             /* a pointer back to the jpeg object */
 } my_source_mgr;
 
 struct my_error_mgr {
    struct jpeg_error_mgr pub;    /* "public" fields */
    jmp_buf setjmp_buffer;        /* for return to caller */
 };
-typedef struct my_error_mgr * my_error_ptr;
+typedef struct my_error_mgr *my_error_ptr;
 
-typedef struct DilloJpeg {
+struct DilloJpeg {
    DilloImage *Image;
    DilloUrl *url;
    int version;
@@ -75,13 +76,12 @@ typedef struct DilloJpeg {
 
    struct jpeg_decompress_struct cinfo;
    struct my_error_mgr jerr;
-} DilloJpeg;
+};
 
 /*
  * Forward declarations
  */
 static void Jpeg_write(DilloJpeg *jpeg, void *Buf, uint_t BufSize);
-METHODDEF(void) Jpeg_errorexit (j_common_ptr cinfo);
 
 
 /* this is the routine called by libjpeg when it detects an error. */
@@ -89,7 +89,12 @@ METHODDEF(void) Jpeg_errorexit (j_common_ptr cinfo)
 {
    /* display message and return to setjmp buffer */
    my_error_ptr myerr = (my_error_ptr) cinfo->err;
-   (*cinfo->err->output_message) (cinfo);
+   if (prefs.show_msg) {
+      DilloJpeg *jpeg =
+                     ((my_source_mgr *) ((j_decompress_ptr) cinfo)->src)->jpeg;
+      MSG_WARN("\"%s\": ", URL_STR(jpeg->url));
+      (*cinfo->err->output_message) (cinfo);
+   }
    longjmp(myerr->setjmp_buffer, 1);
 }
 
@@ -268,8 +273,12 @@ static void Jpeg_write(DilloJpeg *jpeg, void *Buf, uint_t BufSize)
          } else if (jpeg->cinfo.num_components == 3) {
             type = DILLO_IMG_TYPE_RGB;
          } else {
-            _MSG("jpeg: can't handle %d component images\n",
-                 jpeg->cinfo.num_components);
+            MSG("4-component JPEG!\n");
+            if (jpeg->cinfo.jpeg_color_space == JCS_YCCK)
+               MSG("YCCK. Are the colors wrong?\n");
+            if (!jpeg->cinfo.saw_Adobe_marker)
+               MSG("No adobe marker! Is the image shown in reverse video?\n");
+            type = DILLO_IMG_TYPE_CMYK_INV;
          }
          /*
           * If a multiple-scan image is not completely in cache,
@@ -281,9 +290,9 @@ static void Jpeg_write(DilloJpeg *jpeg, void *Buf, uint_t BufSize)
 
          /* check max image size */
          if (jpeg->cinfo.image_width <= 0 || jpeg->cinfo.image_height <= 0 ||
-             jpeg->cinfo.image_width > 
+             jpeg->cinfo.image_width >
              IMAGE_MAX_AREA / jpeg->cinfo.image_height) {
-            MSG("Jpeg_write: suspicious image size request %ux%u\n",
+            MSG("Jpeg_write: suspicious image size request %u x %u\n",
                 (uint_t)jpeg->cinfo.image_width,
                 (uint_t)jpeg->cinfo.image_height);
             jpeg->state = DILLO_JPEG_ERROR;
