@@ -24,6 +24,16 @@
 #include "keys.hh"
 #include "timeout.hh"
 
+/*
+ * Local data types
+ */
+
+typedef struct {
+   const char *title;
+   const Fl_Menu_Item *picked;
+   const Fl_Menu_Item *menu;
+} Menu_popup_data_t;
+
 /*
  * Local data
  */
@@ -68,55 +78,52 @@ static void filemenu_cb(Fl_Widget*, void *data)
 }
 
 
-static void Menu_copy_urlstr_cb(Fl_Widget*, void*)
+static void Menu_copy_urlstr_cb(Fl_Widget*, void *user_data)
 {
-   if (popup_url)
-      a_UIcmd_copy_urlstr(popup_bw, URL_STR(popup_url));
-}
-
-static void Menu_link_cb(Fl_Widget*, void *user_data)
-{
-   DilloUrl *url = (DilloUrl *) user_data ;
-   _MSG("Menu_link_cb: click! :-)\n");
-
-   if (url)
-      a_Menu_link_popup(popup_bw, url);
+   if (user_data) {
+      DilloUrl *url = (DilloUrl *)user_data ;
+      a_UIcmd_copy_urlstr(popup_bw, URL_STR(url));
+   }
 }
 
 /*
  * Open URL
  */
-static void Menu_open_url_cb(Fl_Widget*, void*)
+static void Menu_open_url_cb(Fl_Widget*, void *user_data)
 {
+   DilloUrl *url = (DilloUrl *)user_data;
    _MSG("Open URL cb: click! :-)\n");
-   a_UIcmd_open_url(popup_bw, popup_url);
+   a_UIcmd_open_url(popup_bw, url);
 }
 
 /*
  * Open URL in new window
  */
-static void Menu_open_url_nw_cb(Fl_Widget*, void*)
+static void Menu_open_url_nw_cb(Fl_Widget*, void *user_data)
 {
+   DilloUrl *url = (DilloUrl *)user_data;
    _MSG("Open URL in new window cb: click! :-)\n");
-   a_UIcmd_open_url_nw(popup_bw, popup_url);
+   a_UIcmd_open_url_nw(popup_bw, url);
 }
 
 /*
  * Open URL in new Tab
  */
-static void Menu_open_url_nt_cb(Fl_Widget*, void*)
+static void Menu_open_url_nt_cb(Fl_Widget*, void *user_data)
 {
+   DilloUrl *url = (DilloUrl *)user_data;
    int focus = prefs.focus_new_tab ? 1 : 0;
    if (Fl::event_state(FL_SHIFT)) focus = !focus;
-   a_UIcmd_open_url_nt(popup_bw, popup_url, focus);
+   a_UIcmd_open_url_nt(popup_bw, url, focus);
 }
 
 /*
  * Add bookmark
  */
-static void Menu_add_bookmark_cb(Fl_Widget*, void*)
+static void Menu_add_bookmark_cb(Fl_Widget*, void *user_data)
 {
-   a_UIcmd_add_bookmark(popup_bw, popup_url);
+   DilloUrl *url = (DilloUrl *)user_data;
+   a_UIcmd_add_bookmark(popup_bw, url);
 }
 
 /*
@@ -130,9 +137,10 @@ static void Menu_find_text_cb(Fl_Widget*, void*)
 /*
  * Save link
  */
-static void Menu_save_link_cb(Fl_Widget*, void*)
+static void Menu_save_link_cb(Fl_Widget*, void *user_data)
 {
-   a_UIcmd_save_link(popup_bw, popup_url);
+   DilloUrl *url = (DilloUrl *)user_data;
+   a_UIcmd_save_link(popup_bw, url);
 }
 
 /*
@@ -146,9 +154,10 @@ static void Menu_save_page_cb(Fl_Widget*, void*)
 /*
  * View current page source
  */
-static void Menu_view_page_source_cb(Fl_Widget*, void*)
+static void Menu_view_page_source_cb(Fl_Widget*, void *user_data)
 {
-   a_UIcmd_view_page_source(popup_bw, popup_url);
+   DilloUrl *url = (DilloUrl *)user_data;
+   a_UIcmd_view_page_source(popup_bw, url);
 }
 
 /*
@@ -207,8 +216,20 @@ static void Menu_form_hiddens_cb(Fl_Widget*, void *user_data)
 
 static void Menu_stylesheet_cb(Fl_Widget*, void *vUrl)
 {
+   int mb = Fl::event_button();
    const DilloUrl *url = (const DilloUrl *) vUrl;
-   a_UIcmd_open_url(popup_bw, url);
+
+   if (mb == 1) {
+      a_UIcmd_open_url(popup_bw, url);
+   } else if (mb == 2) {
+      if (prefs.middle_click_opens_new_tab) {
+         int focus = prefs.focus_new_tab ? 1 : 0;
+         if (Fl::event_state(FL_SHIFT)) focus = !focus;
+         a_UIcmd_open_url_nt(popup_bw, url, focus);
+      } else {
+         a_UIcmd_open_url_nw(popup_bw, url);
+      }
+   }
 }
 
 /*
@@ -276,15 +297,35 @@ static void Menu_history_cb(Fl_Widget*, void *data)
  * avoid a segfault when a new page replaces the page that issued
  * the popup menu.
  */
-static void Menu_popup_cb(void *data)
+static void Menu_simple_popup_cb(void *data)
 {
-   const Fl_Menu_Item *m = ((Fl_Menu_Item *)data)->popup(popup_x, popup_y);
+   const Fl_Menu_Item *m;
+
+   ((UI*)popup_bw->ui)->window()->cursor(FL_CURSOR_DEFAULT);
+
+   m = ((Fl_Menu_Item *)data)->popup(popup_x, popup_y);
 
    if (m && m->callback())
       m->do_callback((Fl_Widget *)data);
    a_Timeout_remove();
 }
 
+static void Menu_popup_cb(void *data)
+{
+   const Fl_Menu_Item *picked;
+   Menu_popup_data_t *d = (Menu_popup_data_t *)data;
+
+   ((UI*)popup_bw->ui)->window()->cursor(FL_CURSOR_DEFAULT);
+
+   picked = d->menu->popup(popup_x, popup_y, d->title, d->picked);
+   if (picked) {
+      d->picked = picked;
+      if (picked->callback())
+         picked->do_callback((Fl_Widget *)(d->menu));
+   }
+   a_Timeout_remove();
+}
+
 /*
  * Page popup menu (construction & popup)
  */
@@ -306,6 +347,7 @@ void a_Menu_page_popup(BrowserWindow *bw, const DilloUrl *url,
       {"Save page as...", 0, Menu_save_page_cb,0,0,0,0,0,0},
       {0,0,0,0,0,0,0,0,0}
    };
+   static Menu_popup_data_t page_data = {"Page menu", NULL, pm};
 
    popup_x = Fl::event_x();
    popup_y = Fl::event_y();
@@ -315,10 +357,13 @@ void a_Menu_page_popup(BrowserWindow *bw, const DilloUrl *url,
 
    has_bugs == TRUE ? pm[1].activate() : pm[1].deactivate();
 
-   if (strncmp(URL_STR(url), "dpi:/vsource/", 13) == 0)
+   if (dStrAsciiCasecmp(URL_SCHEME(url), "dpi") == 0 &&
+       strncmp(URL_PATH(url), "/vsource/", 9) == 0)
       pm[0].deactivate();
-   else
+   else {
       pm[0].activate();
+      pm[0].user_data(popup_url);
+   }
 
    if (stylesheets) {
       while (stylesheets[j].text) {
@@ -335,23 +380,20 @@ void a_Menu_page_popup(BrowserWindow *bw, const DilloUrl *url,
       memset(stylesheets, '\0', sizeof(Fl_Menu_Item[cssUrls->size() + 1]));
 
       for (j = 0; j < cssUrls->size(); j++) {
-
-         /* may want ability to Load individual unloaded stylesheets as well */
-         const char *action = "View ";
          DilloUrl *url = cssUrls->get(j);
          const char *url_str = URL_STR(url);
          const uint_t head_length = 30, tail_length = 40,
-                      url_len = strlen(url_str);;
+                      url_len = strlen(url_str);
          char *label;
 
          if (url_len > head_length + tail_length + 3) {
             /* trim long URLs when making the label */
             char *url_head = dStrndup(url_str, head_length);
             const char *url_tail = url_str + (url_len - tail_length);
-            label = dStrconcat(action, url_head, "...", url_tail, NULL);
+            label = dStrconcat(url_head, "...", url_tail, NULL);
             dFree(url_head);
          } else {
-            label = dStrconcat(action, url_str, NULL);
+            label = dStrdup(url_str);
          }
 
          stylesheets[j].label(FL_NORMAL_LABEL, label);
@@ -363,8 +405,27 @@ void a_Menu_page_popup(BrowserWindow *bw, const DilloUrl *url,
    } else {
       pm[2].deactivate();
    }
+   pm[3].user_data(popup_url);
 
-   a_Timeout_add(0.0, Menu_popup_cb, (void*)pm);
+   a_Timeout_add(0.0, Menu_popup_cb, (void*)&page_data);
+}
+
+static Fl_Menu_Item link_menu[] = {
+   {"Open link in new tab", 0, Menu_open_url_nt_cb,0,0,0,0,0,0},
+   {"Open link in new window", 0, Menu_open_url_nw_cb,0,FL_MENU_DIVIDER,0,0,
+    0,0},
+   {"Bookmark this link", 0, Menu_add_bookmark_cb,0,0,0,0,0,0},
+   {"Copy link location", 0, Menu_copy_urlstr_cb,0,FL_MENU_DIVIDER,0,0,0,0},
+   {"Save link as...", 0, Menu_save_link_cb,0,0,0,0,0,0},
+   {0,0,0,0,0,0,0,0,0}
+};
+
+static void Menu_set_link_menu_user_data(void *user_data)
+{
+   int i;
+
+   for (i = 0; link_menu[i].label(); i++)
+      link_menu[i].user_data(user_data);
 }
 
 /*
@@ -372,15 +433,7 @@ void a_Menu_page_popup(BrowserWindow *bw, const DilloUrl *url,
  */
 void a_Menu_link_popup(BrowserWindow *bw, const DilloUrl *url)
 {
-   static Fl_Menu_Item pm[] = {
-      {"Open link in new tab", 0, Menu_open_url_nt_cb,0,0,0,0,0,0},
-      {"Open link in new window", 0, Menu_open_url_nw_cb,0,FL_MENU_DIVIDER,0,0,
-       0,0},
-      {"Bookmark this link", 0, Menu_add_bookmark_cb,0,0,0,0,0,0},
-      {"Copy link location", 0, Menu_copy_urlstr_cb,0,FL_MENU_DIVIDER,0,0,0,0},
-      {"Save link as...", 0, Menu_save_link_cb,0,0,0,0,0,0},
-      {0,0,0,0,0,0,0,0,0}
-   };
+   static Menu_popup_data_t link_data = {"Link menu", NULL, link_menu};
 
    popup_x = Fl::event_x();
    popup_y = Fl::event_y();
@@ -388,7 +441,9 @@ void a_Menu_link_popup(BrowserWindow *bw, const DilloUrl *url)
    a_Url_free(popup_url);
    popup_url = a_Url_dup(url);
 
-   a_Timeout_add(0.0, Menu_popup_cb, (void*)pm);
+   Menu_set_link_menu_user_data(popup_url);
+
+   a_Timeout_add(0.0, Menu_popup_cb, (void*)&link_data);
 }
 
 /*
@@ -409,9 +464,10 @@ void a_Menu_image_popup(BrowserWindow *bw, const DilloUrl *url,
       {"Bookmark this image", 0, Menu_add_bookmark_cb,0,0,0,0,0,0},
       {"Copy image location", 0,Menu_copy_urlstr_cb,0,FL_MENU_DIVIDER,0,0,0,0},
       {"Save image as...", 0, Menu_save_link_cb, 0, FL_MENU_DIVIDER,0,0,0,0},
-      {"Link menu", 0, Menu_link_cb,0,0,0,0,0,0},
+      {"Link menu", 0, Menu_nop_cb, link_menu, FL_SUBMENU_POINTER,0,0,0,0},
       {0,0,0,0,0,0,0,0,0}
    };
+   static Menu_popup_data_t image_data = {"Image menu", NULL, pm};
 
    popup_x = Fl::event_x();
    popup_y = Fl::event_y();
@@ -424,6 +480,10 @@ void a_Menu_image_popup(BrowserWindow *bw, const DilloUrl *url,
    popup_link_url = a_Url_dup(link_url);
 
 
+   pm[0].user_data(popup_url);
+   pm[1].user_data(popup_url);
+   pm[2].user_data(popup_url);
+
    if (loaded_img) {
       pm[3].deactivate();
    } else {
@@ -431,14 +491,18 @@ void a_Menu_image_popup(BrowserWindow *bw, const DilloUrl *url,
       pm[3].user_data(popup_page_url);
    }
 
+   pm[4].user_data(popup_url);
+   pm[5].user_data(popup_url);
+   pm[6].user_data(popup_url);
+
    if (link_url) {
       pm[7].activate();
-      pm[7].user_data(popup_link_url);
+      Menu_set_link_menu_user_data(popup_link_url);
    } else {
       pm[7].deactivate();
    }
 
-   a_Timeout_add(0.0, Menu_popup_cb, (void*)pm);
+   a_Timeout_add(0.0, Menu_popup_cb, (void*)&image_data);
 }
 
 /*
@@ -454,6 +518,7 @@ void a_Menu_form_popup(BrowserWindow *bw, const DilloUrl *page_url,
       {0, 0, Menu_form_hiddens_cb, &hiddens_visible, 0,0,0,0,0},
       {0,0,0,0,0,0,0,0,0}
    };
+   static Menu_popup_data_t form_data = {"Form menu", NULL, pm};
 
    popup_x = Fl::event_x();
    popup_y = Fl::event_y();
@@ -465,7 +530,7 @@ void a_Menu_form_popup(BrowserWindow *bw, const DilloUrl *page_url,
    hiddens_visible = hidvis;
    pm[2].label(hiddens_visible ? "Hide hiddens": "Show hiddens");
 
-   a_Timeout_add(0.0, Menu_popup_cb, (void*)pm);
+   a_Timeout_add(0.0, Menu_popup_cb, (void*)&form_data);
 }
 
 /*
@@ -498,7 +563,7 @@ void a_Menu_file_popup(BrowserWindow *bw, void *v_wid)
    popup_url = NULL;
 
    //pm->label(wid->visible() ? NULL : "File");
-   a_Timeout_add(0.0, Menu_popup_cb, (void*)pm);
+   a_Timeout_add(0.0, Menu_simple_popup_cb, (void*)pm);
 }
 
 /*
@@ -520,7 +585,7 @@ void a_Menu_bugmeter_popup(BrowserWindow *bw, const DilloUrl *url)
    a_Url_free(popup_url);
    popup_url = a_Url_dup(url);
 
-   a_Timeout_add(0.0, Menu_popup_cb, (void*)pm);
+   a_Timeout_add(0.0, Menu_simple_popup_cb, (void*)pm);
 }
 
 /*
@@ -557,7 +622,7 @@ void a_Menu_history_popup(BrowserWindow *bw, int x, int y, int direction)
       pm[i].label(FL_NORMAL_LABEL, a_History_get_title(history_list[i], 1));
       pm[i].callback(Menu_history_cb, INT2VOIDP(i+1));
    }
-   a_Timeout_add(0.0, Menu_popup_cb, (void*)pm);
+   a_Timeout_add(0.0, Menu_simple_popup_cb, (void*)pm);
 }
 
 /*
