@@ -11,28 +11,17 @@
 #define _MSG_WARN(...)
 #define _MSG_HTTP(...)
 
-
-#define MSG(...)                                   \
+#define MSG_INNARDS(prefix, ...)                   \
    D_STMT_START {                                  \
       if (prefs.show_msg){                         \
-         printf(__VA_ARGS__);                      \
+         printf(prefix __VA_ARGS__);               \
          fflush (stdout);                          \
       }                                            \
    } D_STMT_END
 
-#define MSG_WARN(...)                              \
-   D_STMT_START {                                  \
-      if (prefs.show_msg)                          \
-         printf("** WARNING **: " __VA_ARGS__);    \
-   } D_STMT_END
-
-#define MSG_ERR(...)                               \
-   D_STMT_START {                                  \
-      if (prefs.show_msg)                          \
-         printf("** ERROR **: " __VA_ARGS__);      \
-   } D_STMT_END
-
-#define MSG_HTTP(...)                              \
-   printf("HTTP warning: " __VA_ARGS__)
+#define MSG(...) MSG_INNARDS("", __VA_ARGS__)
+#define MSG_WARN(...) MSG_INNARDS("** WARNING **: ", __VA_ARGS__)
+#define MSG_ERR(...) MSG_INNARDS("** ERROR **: ", __VA_ARGS__)
+#define MSG_HTTP(...) MSG_INNARDS("HTTP warning: ", __VA_ARGS__)
 
 #endif /* __MSG_H__ */
