@@ -53,20 +53,18 @@ static char *prog_state_name[] =
  * structure below so that processing can be suspended or resumed at any
  * point within an input image.
  *
- * In the case of the libpng library, it maintains it's own state in
+ * In the case of the libpng library, it maintains its own state in
  * png_ptr and into_ptr so the FSM is very simple - much simpler than the
  * ones for XBM and PNM are.
  */
 
-typedef
-struct _DilloPng {
+typedef struct {
    DilloImage *Image;           /* Image meta data */
    DilloUrl *url;               /* Primary Key for the dicache */
    int version;                 /* Secondary Key for the dicache */
 
-   double display_exponent;     /* gamma correction */
-   ulong_t width;               /* png image width */
-   ulong_t height;              /* png image height */
+   png_uint_32 width;           /* png image width */
+   png_uint_32 height;          /* png image height */
    png_structp png_ptr;         /* libpng private data */
    png_infop info_ptr;          /* libpng private info */
    uchar_t *image_data;         /* decoded image data    */
@@ -75,7 +73,6 @@ struct _DilloPng {
    int error;                   /* error flag */
    png_uint_32 previous_row;
    int rowbytes;                /* No. bytes in image row */
-   short passes;
    short channels;              /* No. image channels */
 
 /*
@@ -98,8 +95,6 @@ struct _DilloPng {
 } DilloPng;
 
 #define DATASIZE  (png->ipbufsize - png->ipbufstart)
-#define BLACK     0
-#define WHITE     255
 
 
 static
@@ -135,17 +130,17 @@ Png_datainfo_callback(png_structp png_ptr, png_infop info_ptr)
                 &bit_depth, &color_type, &interlace_type, NULL, NULL);
 
    /* check max image size */
-   if (png->width <= 0 || png->height <= 0 ||
+   if (png->width == 0 || png->height == 0 ||
        png->width > IMAGE_MAX_AREA / png->height) {
-      MSG("Png_datainfo_callback: suspicious image size request %ldx%ld\n",
-          png->width, png->height);
+      MSG("Png_datainfo_callback: suspicious image size request %lu x %lu\n",
+          (ulong_t) png->width, (ulong_t) png->height);
       Png_error_handling(png_ptr, "Aborting...");
       return; /* not reached */
    }
 
-   _MSG("Png_datainfo_callback: png->width  = %ld\n"
-        "Png_datainfo_callback: png->height = %ld\n",
-        png->width, png->height);
+   _MSG("Png_datainfo_callback: png->width  = %lu\n"
+        "Png_datainfo_callback: png->height = %lu\n",
+        (ulong_t) png->width, (ulong_t) png->height);
 
    /* we need RGB/RGBA in the end */
    if (color_type == PNG_COLOR_TYPE_PALETTE && bit_depth <= 8) {
@@ -179,10 +174,10 @@ Png_datainfo_callback(png_structp png_ptr, png_infop info_ptr)
 
    /* Interlaced */
    if (interlace_type != PNG_INTERLACE_NONE) {
-      png->passes = png_set_interlace_handling(png_ptr);
+      png_set_interlace_handling(png_ptr);
    }
 
-   /* get libpng to update it's state */
+   /* get libpng to update its state */
    png_read_update_info(png_ptr, info_ptr);
 
    png_get_IHDR(png_ptr, info_ptr, &png->width, &png->height,
@@ -193,9 +188,9 @@ Png_datainfo_callback(png_structp png_ptr, png_infop info_ptr)
 
    /* init Dillo specifics */
    _MSG("Png_datainfo_callback: rowbytes = %d\n"
-        "Png_datainfo_callback: width    = %ld\n"
-        "Png_datainfo_callback: height   = %ld\n",
-        png->rowbytes, png->width, png->height);
+        "Png_datainfo_callback: width    = %lu\n"
+        "Png_datainfo_callback: height   = %lu\n",
+        png->rowbytes, (ulong_t) png->width, (ulong_t) png->height);
 
    png->image_data = (uchar_t *) dMalloc(png->rowbytes * png->height);
    png->row_pointers = (uchar_t **) dMalloc(png->height * sizeof(uchar_t *));
@@ -345,8 +340,7 @@ static void Png_write(DilloPng *png, void *Buf, uint_t BufSize)
             return;            /* need MORE data */
          }
          /* check the image signature - DON'T update ipbufstart! */
-         if (!png_check_sig(png->ipbuf, DATASIZE)) {
-            /* you lied to me about it being a PNG image */
+         if (png_sig_cmp(png->ipbuf, 0, DATASIZE)) {
             MSG_WARN("\"%s\" is not a PNG file.\n", URL_STR(png->url));
             png->state = IS_finished;
             break;
