@@ -688,15 +688,15 @@ static uint_t Url_host_public_internal_dots(const char *host)
 
       if (tld_len > 0) {
          /* These TLDs were chosen by examining the current publicsuffix list
-          * in September 2011 and picking out those where it was simplest for
-          * them to describe the situation by beginning with a "*.[tld]" rule.
+          * in October 2012 and picking out those where it was simplest for
+          * them to describe the situation by beginning with a "*.[tld]" rule
+          * or every rule was "[something].[tld]".
           */
-         const char *const tlds[] = {"ar","au","bd","bn","bt","ck","cy",
-                                     "er","et","fj","fk","gt","gu","id",
-                                     "il","jm","ke","kh","kw","ml","mm","mt",
-                                     "mz","ni","np","nz","om","pg","py","qa",
-                                     "sv","tr","uk","uy","ve","ye","yu","za",
-                                     "zm","zw"};
+         const char *const tlds[] = {"ar","au","bd","bn","ck","cy","er","et",
+                                     "fj","fk","gt","gu","il","jm","ke","kh",
+                                     "kp","kw","lb","lr","mm","mt","mz","ni",
+                                     "np","nz","om","pg","py","sv","tr","uk",
+                                     "ve","ye","za","zm","zw"};
          uint_t i, tld_num = sizeof(tlds) / sizeof(tlds[0]);
 
          for (i = 0; i < tld_num; i++) {
